
/*===========================================================================*/
/* SEED reader     |              output_sac_ascii         |    subprocedure */
/*===========================================================================*/
/*
	Name:	output_sac_ascii
	Purpose:build and write a seismogram file from the SEED header tables
		and data files; serve as an example for users who wish to write
		their own "output_data" procedure.  This procedure writes SAC
		files.
	Usage:	void output_sac_ascii();

	Input:	none (gets its data from globally-available tables and files)
	Output:	none (writes a seismogram file; seismogram files are named by
		beginning time, station, and component; for example,
		1988.023.15.34.08.2800.ANMO.SPZ is the filename for a
		seismogram from year 1988, Julian day 23, 15:34:08.2800 UT,
		recorded at station ANMO from component SPZ.  Writes message
		to stderr describing the seismogram file being written.
	Externals:data_hdr - a table containing information about this seismogram
		current_station - a pointer to the SEED header tables for the
		station at which this seismogram was recorded
		current_channel - a pointer to the SEED header tables for the
		channel of the station at which this seismogram was recorded
	Warnings:unable to open seismogram file for writing
		failure to properly write the seismic data
	Errors:	none
	Called by:time_span_out
	Calls to:none
	Algorithm:build the seismogram file header by copying values from the
			structure "data_hdr" and the "current_station" and
			"current_channel" tables.  Write out the header and the data.
			This procedure may write either ASCII or binary data, depending
			upon the values of the variables "ascii" and "binary"; as
			implemented here, the programmer sets "ascii" to TRUE or FALSE
			and "binary" is automatically set to the negation of "ascii".
	Notes:	"output_data" is the name of the process called by
		"process_data" to write a seismogram file.  In the case shown
		here, the data are written in SAC format; the SAC format was
		chosen because many users have SAC available and will be able to
		use the output files directly.  If the user desires a different
		output format, this code and these notes should contain enough
		information for that user to construct his or her own 
		"output_data" procedure.

		The structure "data_hdr" contains the following information from
		the SEED fixed-length data header:

	structure element                     description
	====================================  ==========================
	struct input_data_hdr                 fixed data header
	{
		char station[5];                   station name
		char location[2];                  station location
		char channel[3];                   channel name
		char unused[2];                    not used
		struct input_time time;            time (see below)
		unsigned short int nsamples;       number samples
		short int sample_rate;             sample rate factor
		short int sample_rate_multiplier;  sample rate multiplier
		char activity_flags;               activity flags
		char io_flags;                     i/o flags
		char data_quality_flags;           data quality flags
		char number_blockettes;            # blockettes which follow
		long int number_time_corrections;  # .0001s time corrections
		unsigned short int bod;            beginning of data
		unsigned short int bofb;           beginning 1st blkt
	};
	struct input_time                     time from input stream
	{
		unsigned short int year;           year
		unsigned short int day;            Julian day
		char hour;                         hour
		char minute;                       minute
		char second;                       second
		char unused;
		unsigned short int fracsec;        10000ths of second
	};

	These header data are available to the programmer in the usual
	ways.  In the case shown here, a SAC header structure was
	defined in "sac.h" and various elements of the SAC header were
	filled in with data available from the SEED fixed-length data
	header.  See the section of code entitled "Build the SAC header"
	for examples.

	Other data are available from the tables pointed to by
	"current_station" and "current_channel".  These data are 
	described in Halbert et al. under the heading "Station Control
	Header Format".  The table structure is described in the leading
	comments to the main program ("rdseed.c") and in the 
	programmer's technical reference manual; the pointers
	"current_station" and "current_channel" are set by the procedure
	"process_data".  See the section of code within "Build the SAC 
	header" commented with "give values available from SEED tables 
	to SAC optional variables" for examples of the use of these
	station- and channel-specific items.

	Numeric data formats included here are correct for SAC.  The
	user may have to change them for output of data to be used by
	other seismic processing programs.
			
	Problems:	none known
	References:	Halbert, S. E., R. Buland, and C. R. Hutt (1988).  Standard for
			the Exchange of Earthquake Data (SEED), Version V2.0,
			February 25, 1988.  United States Geological Survey,
			Albuquerque Seismological Laboratory, Building 10002,
			Kirtland Air Force Base East, Albuquerque, New Mexico
			87115.  82 pp.
		O'Neill, D. (1987).  IRIS Interim Data Distribution Format
		(SAC ASCII), Version 1.0 (12 November 1987).  Incorporated
		Research Institutions for Seismology, 1616 North Fort Myer
		Drive, Suite 1440, Arlington, Virginia 22209.  11 pp.
		Tull, J. (1987).  SAC User's Manual, Version 10.2, October 7,
		1987.  Lawrence Livermore National Laboratory, L-205,
		Livermore, California 94550.  ??? pp.
	Language:	C, hopefully ANSI standard
	Author:		Dennis O'Neill
	Revisions:	07/15/88  Dennis O'Neill  Initial preliminary release 0.9
		11/21/88  Dennis O'Neill  Production release 1.0
		09/19/89  Dennis O'Neill  corrected output string length
		05/01/92  Allen Nance  corrected CMPINC Header variable for SEED Dip definition
		sach.cmpinc = current_channel->dip + 90.0;
*/

#include <stdio.h>
#include <sys/param.h>

#include "rdseed.h"	/* SEED tables and structures */
#include "sac.h"	/* SAC format structures */


extern int Output_PnZs;
extern int EventDirFlag;

char *summary_file_get_event();
char *get_event_dir_name();


struct type53 *find_type_53();
struct type43 *find_type_43();
struct type34 *find_type_34();
float get_A0();
 
char strlst();

void output_sac_ascii(data_hdr, offset)
struct data_hdr *data_hdr;
int offset;
{
	FILE *outfile;			/* output file pointer */
	char outfile_name[100], channel[10];	/* output file name */
	struct sac sach;			/* SAC header structure */
	int i, j,k;				/* counter */
	char character[8+1];		/* for character transfer */
	int ascii;			/* switch for ASCII output */
	int binary;			/* switch for binary output */
	int reverse_flag;		/* Data reversal flag */
	double dip, azimuth;		/* temp holding for these variables */

	char orig_dir[MAXPATHLEN];

	getcwd(orig_dir, MAXPATHLEN);

	/* initialize SAC Header to all nulls */
	sach = sac_null;

	for (k=0;k<data_hdr->num_mux_chan;k++)
	{
		if (current_channel == NULL)
		{
			fprintf(stderr, "\tERROR - Unexpected End of Multiplexed Channel List\n");
			return;
		}

		reverse_flag = 0;
		for (i=0;i<10;i++) 
			channel[i] = 0;

		strcpy(channel,current_channel->channel);

        	if (data_hdr->num_mux_chan > 1) 
		{
                	int i;

                	for (i=0; i<3; i++) {
                        	if (channel[i] == ' ' || 
				    channel[i] == '\0') 
				break;
                }

                	if (i > 2) 
				i = 2;

                	channel[i] = data_hdr->mux_chan_name[k];
        	}

		dip = current_channel->dip;
		azimuth = current_channel->azimuth;

	switch (strlst(channel))
	{
		case 'Z':
			if (dip==90.0 && azimuth==0.0)
				reverse_flag = 1;
			break;
		case 'N':
			if (dip==0.0 && azimuth==180.0)
				reverse_flag = 1;
			break;
		case 'E':
			if (dip==0.0 && azimuth==270.0)
				reverse_flag = 1;
			break;
		case 'A':
			if (dip==60.0 && azimuth==180.0)
				reverse_flag = 1;
			break;
		case 'B':
			if (dip==60.0 && azimuth==300.0)
				reverse_flag = 1;
			break;
		case 'C':
			if (dip==60.0 && azimuth==60.0)
				reverse_flag = 1;
			break;
		case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9': case '0': case 'O':
			break;
		default:
			fprintf(stderr, "Warning... Invalid Channel Direction character on %s,%s\n", current_station->station, channel);
			break; 
	}

	{
	struct response *response; int flag;

	flag = FALSE;
	for (response = current_channel->response_head; response != NULL; response = response->next)
	{
		reverse_flag &= 1;
		if (response->type == 'S') 
		if ((response->ptr.type58)->stage == 0)
		{
			if (((response->ptr.type58)->sensitivity < 0.0) && 
				(check_reverse & 2))
				reverse_flag |= 2;

			sach.scale = (float)(response->ptr.type58)->sensitivity;
			flag = TRUE;
		}
	}
	if (!flag && (check_reverse & 2))
		fprintf(stderr, "Warning - Stage Zero Not Found on Gain Reversal check\n");
	}
 
	if (check_reverse & 1)
	{
		if (reverse_flag & 1)
			fprintf (stderr, "Warning... Azimuth/Dip Reversal found %s.%s, Data will be automatically inverted\n           Response Information will not be modified\n",
			current_station->station, channel);
	}
	else
	{
		if (reverse_flag & 1)
			fprintf (stderr, "Warning... Azimuth/Dip Reversal found %s.%s, Data inversion was not selected\n",
				current_station->station, channel);
		reverse_flag &= 2;
	}

	if (check_reverse & 2)
	{
		if (reverse_flag & 2)
			fprintf (stderr, "Warning... Gain Reversal found %s.%s, Data will be automatically inverted\n           Response Information will not be modified\n",
				current_station->station, channel);
	}
	else
	{
		if (reverse_flag & 2)
			fprintf (stderr, "Warning... Gain Reversal found %s.%s, Data inversion was not selected\n",
				current_station->station, channel);
		reverse_flag &= 1;
	}

	if (reverse_flag == 3)
	{
		reverse_flag = 0;		/* Double inversion */
		fprintf (stderr, "Warning... Gain and Dip/Azimuth Reversal found %s.%s, Data was not inverted\n",
			current_station->station, channel);
	}

	if (reverse_flag & 1)
	switch (strlst(channel))
	{
		case 'Z':
			dip = -90.0;
			break;
		case 'N':
			azimuth = 0.0;
			break;
		case 'E':
			azimuth = 90.0;
			break;
		case 'A':
			dip = -60.0;
			azimuth = 0.0;
			break;
		case 'B':
			dip = 60.0;
			azimuth = 120.0;
			break;
		case 'C':
			dip = -60.0;
			azimuth = 240.0;
			break;
		default:
			break; 
	}

	if (reverse_flag & 2) sach.scale = -sach.scale;

/*                 +=======================================+                 */
/*=================|          Build the SAC header         |=================*/
/*                 +=======================================+                 */


	if (data_hdr->nsamples == 0)
	{
		fprintf(stderr,"\tWARNING (output_sac):  Output Data Contains No Samples\n");
		chdir(orig_dir);
		return;
	}

/* give values to SAC-required variables */
	/* give values to SAC-required variables */
	sach.delta = 1 / data_hdr->sample_rate;
	sach.b = 0.0;
	sach.e = ((double) data_hdr->nsamples - 1) / ((double) data_hdr->sample_rate);
	sach.npts = data_hdr->nsamples;
	sach.iftype = ITIME;			/* known a priori */
	sach.leven = TRUE;			/* known a priori */

	/* give values available from struct data_hdr to SAC optional variables */
	/* record start time */
	sach.nzyear = data_hdr->time.year;
	sach.nzjday = data_hdr->time.day;
	sach.nzhour = data_hdr->time.hour;
	sach.nzmin  = data_hdr->time.minute;
	sach.nzsec  = data_hdr->time.second;
	sach.nzmsec = data_hdr->time.fracsec / 10;	/* convert to msec */

	/* station and component names */
	sprintf (character, "%-8.8s", data_hdr->station);
	strncpy (sach.kstnm, character, 8);
	sprintf (character, "%-8.8s", channel);
	strncpy (sach.kcmpnm, character, 8);

	/* give values available from SEED tables to SAC optional variables */
	sach.stla = current_channel->latitude;
	sach.stlo = current_channel->longitude;
	sach.stel = current_channel->elevation;
	sach.stdp = current_channel->local_depth;

	sach.cmpaz = azimuth;
	sach.cmpinc = dip + 90.0; /* SEED dip is 90 degrees off from SAC dip */

	/* event lat/lon from hypo B071 */
	if (type71_head != NULL)
	{
		int error = 0;

		sach.evla = type71_head->latitude;
		sach.evlo = type71_head->longitude;
		sach.evdp = type71_head->depth;

		
		error = delaz(sach.evla, sach.evlo,
                                  sach.stla, sach.stlo,
                                      &sach.gcarc, &sach.az, &sach.baz);


		if (error)
		{
			fprintf(stderr, "Error - unable to compute event distance/azimuth and back azimuth for station %s; channel %s!\n", 
					current_station->station, 
					current_channel->channel);

			sach.gcarc = 0; sach.az = 0; sach.baz = 0;
		}
		else
			/* 111.1 KM per Degree - gcarc is in degrees */
			sach.dist = 111.1 * sach.gcarc;
			
	}

	/* mysterious LLL-set values */
	sach.internal1 = 2.0;
	sach.internal4 = 6;
	sach.internal5 = 0;
	sach.internal6 = 0;
	sach.unused27 = FALSE;

	/* not sure if these are needed, but they might be. Values might be wrong */
	sach.lpspol = FALSE;
	sach.lcalda = TRUE;

/*                 +=======================================+                 */
/*=================|  build name for and open output file  |=================*/
/*                 +=======================================+                 */

	/* 1988.023.15.34.08.2800.ANMO.SPZ is the filename for a seismogram 
  	 *	from year 1988, Julian day 23, 15:34:08.2800 UT, recorded 
	 *	at station ANMO from component SPZ. 
	 */

	/* if requested from WEED, place into subdirectory */
	if (EventDirFlag)
	{
		char dirname[MAXPATHLEN];

		strcpy(dirname, get_event_dir_name());

		if (!dir_exists(dirname))
                {
                        if (mkdir(dirname, 0777) == -1)
                        {
                                fprintf(stderr, "Unable to create event directory. Using current directory!\n");
                                perror("output_sac");
                                strcpy(dirname, "./");
                        }
 
                }
		
		chdir(dirname);
 
	}

	sprintf (outfile_name, 
			"%04d.%03d.%02d.%02d.%02d.%04d.%s.%s.%s.%s.SAC_ASC",
		data_hdr->time.year,
		data_hdr->time.day,
		data_hdr->time.hour,
		data_hdr->time.minute,
		data_hdr->time.second,
		data_hdr->time.fracsec,
		type10.version >= 2.3 ? data_hdr->network : "",
		data_hdr->station,
		data_hdr->location,
		channel);

	if ((outfile = fopen (outfile_name, "w+")) == NULL)
	{
		fprintf (stderr, "\tWARNING (output_sac):  ");
		fprintf (stderr, "Output file %s is not available for writing.\n",
			outfile_name);
		fprintf (stderr, "\tExecution continuing.\n");

		chdir(orig_dir);
		
		return;
	}

/*                 +=======================================+                 */
/*=================|        write a SAC ASCII file         |=================*/
/*                 +=======================================+                 */

		/* describe the file being written */
		printf ("Writing %s, %s, %s, %5d samples (ASCII),",
			data_hdr->station,
			data_hdr->location,
			channel,
			data_hdr->nsamples);

		printf (" starting %04d,%03d %02d:%02d:%02d.%04d UT\n",
			data_hdr->time.year,
			data_hdr->time.day,
			data_hdr->time.hour,
			data_hdr->time.minute,
			data_hdr->time.second,
			data_hdr->time.fracsec);

		/* write the SAC header */
		fprintf (outfile, FCS, 
			sach.delta, 
			sach.depmin, 
			sach.depmax, 
			sach.scale, 
			sach.odelta);
		fprintf (outfile, FCS, 
			sach.b, 
			sach.e, 
			sach.o, 
			sach.a, 
			sach.internal1);
		fprintf (outfile, FCS, 
			sach.t0, 
			sach.t1, 
			sach.t2, 
			sach.t3, 
			sach.t4);
		fprintf (outfile, FCS, 
			sach.t5, 
			sach.t6, 
			sach.t7, 
			sach.t8, 
			sach.t9);
		fprintf (outfile, FCS, 
			sach.f, 
			sach.resp0, 
			sach.resp1, 
			sach.resp2, 
			sach.resp3);
		fprintf (outfile, FCS, 
			sach.resp4, 
			sach.resp5, 
			sach.resp6, 
			sach.resp7, 
			sach.resp8);
		fprintf (outfile, FCS, 
			sach.resp9, 
			sach.stla, 
			sach.stlo, 
			sach.stel, 
			sach.stdp);
		fprintf (outfile, FCS, 
			sach.evla,
			sach.evlo,
			sach.evel,
			sach.evdp,
			sach.unused1);
		fprintf (outfile, FCS, 
			sach.user0,
			sach.user1,
			sach.user2,
			sach.user3,
			sach.user4);
		fprintf (outfile, FCS, 
			sach.user5,
			sach.user6,
			sach.user7,
			sach.user8,
			sach.user9);
		fprintf (outfile, FCS, 
			sach.dist,
			sach.az,
			sach.baz,
			sach.gcarc,
			sach.internal2);
		fprintf (outfile, FCS, 
			sach.internal3,
			sach.depmen,
			sach.cmpaz,
			sach.cmpinc,
			sach.unused2);
		fprintf (outfile, FCS, 
			sach.unused3,
			sach.unused4,
			sach.unused5,
			sach.unused6,
			sach.unused7);
		fprintf (outfile, FCS, 
			sach.unused8,
			sach.unused9,
			sach.unused10,
			sach.unused11,
			sach.unused12);
		fprintf (outfile, ICS, 
			sach.nzyear,
			sach.nzjday,
			sach.nzhour,
			sach.nzmin,
			sach.nzsec);
		fprintf (outfile, ICS, 
			sach.nzmsec,
			sach.internal4,
			sach.internal5,
			sach.internal6,
			sach.npts);
		fprintf (outfile, ICS, 
			sach.internal7,
			sach.internal8,
			sach.unused13,
			sach.unused14,
			sach.unused15);
		fprintf (outfile, ICS, 
			sach.iftype,
			sach.idep,
			sach.iztype,
			sach.unused16,
			sach.iinst);
		fprintf (outfile, ICS, 
			sach.istreg,
			sach.ievreg,
			sach.ievtyp,
			sach.iqual,
			sach.isynth);
		fprintf (outfile, ICS, 
			sach.unused17,
			sach.unused18,
			sach.unused19,
			sach.unused20,
			sach.unused21);
		fprintf (outfile, ICS, 
			sach.unused22,
			sach.unused23,
			sach.unused24,
			sach.unused25,
			sach.unused26);
		fprintf (outfile, ICS, 
			sach.leven,
			sach.lpspol,
			sach.lovrok,
			sach.lcalda,
			sach.unused27);
		fprintf (outfile, CCS2, 
			sach.kstnm,
			sach.kevnm);
		fprintf (outfile, CCS1, 
			sach.khole,
			sach.ko,
			sach.ka);
		fprintf (outfile, CCS1, 
			sach.kt0,
			sach.kt1,
			sach.kt2);
		fprintf (outfile, CCS1, 
			sach.kt3,
			sach.kt4,
			sach.kt5);
		fprintf (outfile, CCS1, 
			sach.kt6,
			sach.kt7,
			sach.kt8);
		fprintf (outfile, CCS1, 
			sach.kt9,
			sach.kf,
			sach.kuser0);
		fprintf (outfile, CCS1, 
			sach.kuser1,
			sach.kuser2,
			sach.kcmpnm);
		fprintf (outfile, CCS1, 
			sach.knetwk,
			sach.kdatrd,
			sach.kinst);

		/* write the SAC seismic data */
		j = k*(seis_buffer_length/data_hdr->num_mux_chan);
		if (reverse_flag)
		{
			char wrkstr[200];

			for (i=0; i<data_hdr->nsamples; i++)
				seismic_data[offset+i+j] = -seismic_data[offset+i+j];

			sprintf(wrkstr, "Data reversal initiated for SAC ASCII file %s", outfile_name);

			rdseed_alert_msg_out(wrkstr);

		}
		for (i=0; i<data_hdr->nsamples; i++)
		{
			fprintf (outfile, "%15.7g", 
					seismic_data[offset+i+j]);

			if ((((i+1)%5) == 0) && (i > 0) ) 
			{
				fprintf (outfile, "\n");
				fflush(outfile);
			}
		}

		fprintf(outfile, "\n");

		fclose(outfile);

		current_channel = current_channel->next;

	}		/* the BIG k loop */

	chdir(orig_dir);
}

/* ------------------------------------------------------------------------- */

