/*===========================================================================*/
/* SEED reader     |             convert_seedhdr           |    process data */
/*===========================================================================*/
/*
	Name:		convert_seedhdr
	Purpose:	build a minimum data header from the SEED fixed section data hdr
				in the same form as tables for easy comparison
	Usage:		void convert_seedhdr ();
				char *input_data_ptr;
				struct input_data_hdr *input_data_hdr;
				input_data_ptr = input.data;
				input_data_hdr = (struct input_data_hdr *) input_data_ptr;
				convert_seedhdr (input_data_hdr);
	Input:		input_data_hdr = a structure containing the SEED fixed section
					of data header information
	Output:		data_hdr = a structure containing minimal header information,
					as follows:
					char *station                    = station name
					char *location                   = station location
					char *channel                    = channel name
					struct time time                 =  time
					int nsamples                     = number of samples
					double sample_rate               = sample rate per second
					char activity_flags              = activity flags
					char io_flags                    = i/o flags
					char data_quality_flags          = data quality flags
					long int number_time_corrections = # .0001s time corrections
	Externals:	input.data - pointer to beginning of input data block
				*data_hdr - defined in structures.h, allocated in globals.h
	Warnings:	none
	Errors:		none
	Called by:	process_data
	Calls to:	none
	Algorithm:	mostly direct cast-and-copy of values from the SEED structure
				"input_data_hdr" to a more easily-used form.  Sample rate is
				calculated according to the rules in the SEED description of the
				fixed section of data header.
	Notes:		none
	Problems:	none known
	References:	Halbert et al, 1988; see main routine
	Language:	Sun OS 3.2 C
	Revisions:	07/15/88  Dennis O'Neill  Preliminary initial release 0.9
*/

#include "rdseed.h"

void convert_seedhdr (input_data_hdr)
struct input_data_hdr *input_data_hdr;
{
	int i;										/* counter */
	char temp_char[5+1];						/* for character transfer */
	double sample_rate;						/* sample rate calculation */
	int sample_rate_multiplier;					/* sample rate calculation */
	struct input_time temptime;					/* temp for byte-swapping */
	short int temp_2byte;						/* temp for byte-swapping */
	long int temp_4byte;						/* temp for byte-swapping */
	unsigned short int temp_u2byte;				/* temp for byte-swapping */

/*                 +=======================================+                 */
/*=================|   create minimum useful data header   |=================*/
/*                 +=======================================+                 */

	/* copy, strip trailing blanks from text strings, put in structure */
	/* copy station name */
	for (i = 0; i < 5; i++) temp_char[i] = input_data_hdr->station[i];
	temp_char[5] = '\0';
	for (i = 4; temp_char[i] == ' '; i--) temp_char[i] = '\0';
	sprintf (data_hdr->station, "%s", temp_char);

	/* copy location code */
	for (i = 0; i < 2; i++) temp_char[i] = input_data_hdr->location[i];
	temp_char[2] = '\0';
	for (i = 1; temp_char[i] == ' '; i--) temp_char[i] = '\0';
	sprintf (data_hdr->location, "%s", temp_char);

	/* copy channel name */
	for (i = 0; i < 3; i++) temp_char[i] = input_data_hdr->channel[i];
	temp_char[3] = '\0';
	for (i = 2; temp_char[i] == ' '; i--) temp_char[i] = '\0';
	sprintf (data_hdr->channel, "%s", temp_char);

	/* copy time information */
	temptime.year    = input_data_hdr->time.year;
	temptime.day     = input_data_hdr->time.day;
	temptime.hour    = input_data_hdr->time.hour;
	temptime.minute  = input_data_hdr->time.minute;
	temptime.second  = input_data_hdr->time.second;
	temptime.fracsec = input_data_hdr->time.fracsec;
	if (byteswap == TRUE)
	{
		temptime.year    = swap_2byte (temptime.year);
		temptime.day     = swap_2byte (temptime.day);
		temptime.fracsec = swap_2byte (temptime.fracsec);
	}   
	data_hdr->time.year    = (int) temptime.year;
	data_hdr->time.day     = (int) temptime.day;
	data_hdr->time.hour    = (int) temptime.hour;
	data_hdr->time.minute  = (int) temptime.minute;
	data_hdr->time.second  = (int) temptime.second;
	data_hdr->time.fracsec = (int) temptime.fracsec;

	/* copy numerical information */
	temp_u2byte = input_data_hdr->nsamples;
	if (byteswap == TRUE) temp_u2byte = swap_2byte (temp_u2byte);
	data_hdr->nsamples = (int) temp_u2byte;

	temp_4byte = input_data_hdr->number_time_corrections;
	if (byteswap == TRUE) temp_4byte = swap_4byte (temp_4byte);
	data_hdr->number_time_corrections = (long int) temp_4byte;

	/* recover sample rate information */
	sample_rate            = (double) input_data_hdr->sample_rate;
	sample_rate_multiplier = (int) input_data_hdr->sample_rate_multiplier;

	temp_2byte = input_data_hdr->sample_rate;
	if (byteswap == TRUE) temp_2byte = swap_2byte (temp_2byte);
	sample_rate = (double) temp_2byte;
 
	temp_2byte = input_data_hdr->sample_rate_multiplier;
	if (byteswap == TRUE) temp_2byte = swap_2byte (temp_2byte);
	sample_rate_multiplier = (int) temp_2byte;

	/* adjust sample rate according to SEED definitions */
	if (sample_rate < 0) sample_rate = 1 / (-sample_rate);
	if (sample_rate_multiplier > 0) 
		sample_rate = sample_rate * sample_rate_multiplier;
	else if (sample_rate_multiplier < 0) 
		sample_rate = sample_rate / (-sample_rate_multiplier);

	/* put sample rate into the structure */
	data_hdr->sample_rate = sample_rate;

	/* copy activity, io, and data quality flags */
	data_hdr->activity_flags     = input_data_hdr->activity_flags;
	data_hdr->io_flags           = input_data_hdr->io_flags;
	data_hdr->data_quality_flags = input_data_hdr->data_quality_flags;

	data_hdr->number_blockettes = (int) input_data_hdr->number_blockettes;

	/* starting byte number of data */
	temp_u2byte = input_data_hdr->bod;
	if (byteswap == TRUE) temp_u2byte = swap_2byte (temp_u2byte);
	data_hdr->bod = (int) temp_u2byte;

	/* starting byte number of blockettes */
	temp_u2byte = input_data_hdr->bofb;
	if (byteswap == TRUE) temp_u2byte = swap_2byte (temp_u2byte);
	data_hdr->bofb = (int) temp_u2byte;

}
